from typing import Dict, Optional

from . import v1 as api_v1
from .api import API, Resource
from .transport import HTTPWireProtocol

supported_versions: Dict[str, API] = {"1": api_v1}  # type: ignore


class UioOnguardClient:
    api: API
    transport: HTTPWireProtocol

    _resource_cache: Dict[str, Resource] = {}

    def __init__(
        self, url: str, headers: Optional[Dict[str, str]] = None, version: str = "1"
    ):
        try:
            self.api = supported_versions[version]
        except KeyError:
            raise ValueError(f"Unsupported API version: {version}")
        self.transport = HTTPWireProtocol(url, headers=headers)

    def __getattr__(self, name):
        try:
            return self._ensure_resource(name)
        except KeyError as err:
            raise AttributeError() from err

    def _ensure_resource(self, ns):
        if ns in self._resource_cache:
            return self._resource_cache[ns]
        cls = self.api.resources[ns]
        self._resource_cache[ns] = cls(transport=self.transport)
        return self._ensure_resource(ns)

    def __str__(self):
        return f"<{self.__class__.__name__} {self.transport}>"
